#define OLC_PGE_APPLICATION
#include "olcPixelGameEngine.h"

#define PI 3.14159f

#include <ranges>

using namespace olc;

struct Rom{
	const static uint_fast16_t MIN_INDEX{};
	const static uint_fast16_t MAX_INDEX{326U};
	std::array<uint_fast32_t,114>paletteBits;
	std::array<uint_fast32_t,103>graphicsBits;
	enum Command{
		UNCOMPRESSED_BLOCK,
		RUN_LENGTH_ENCODED_BYTE,
		RUN_LENGTH_ENCODED_SHORT,
		INCREMENTAL_SEQUENCE,
		REPEAT_PREVIOUS_DATA,
		REVERSE_BITS,
		UNKNOWN_1,
		UNKNOWN_2,
	};

	static uint_fast32_t snesToHex(uint_fast32_t addr,bool header=true){
		uint_fast32_t newAddr{addr};
		if(newAddr>=0x400000&&newAddr<0x600000);//NO-OP
		else if(newAddr>=0xC00000&&newAddr<0x1000000)newAddr-=0xC00000;
		else throw std::out_of_range{std::format("SNES address out of range: {}",newAddr)};

		if(header)newAddr+=0x200;
		return newAddr-0xA0200;
	}
	static uint_fast32_t hexToSnes(uint_fast32_t addr,bool header=true){
		uint_fast32_t newAddr{addr};
		if(header)newAddr-=0x200;
		if(newAddr>=0&&newAddr<0x400000)return newAddr+0xC00000;
		else if(newAddr>=0x400000&&newAddr<0x600000)return newAddr;
		else throw std::out_of_range{std::format("File offset out of range: {}",newAddr)};
	}

	struct DataBlock{
		uint_fast32_t addr,ptr;
		std::u8string_view data;
		DataBlock(std::u8string_view data,uint_fast32_t loc)
		:data(data){
			addr=ptr=loc;
		}
		uint_fast8_t readInt8(){
			return data[ptr++];
		}
		uint_fast16_t readShort(){
			return readInt8()|(readInt8()<<8);
		}
		uint_fast32_t readInt32(){
			return readInt8()|(readInt8()<<8)|(readInt8()<<16)|(readInt8()<<24);
		}
		uint_fast32_t getCompressedSize(uint_fast32_t pointer){
			int32_t bpos{};
			int32_t pos{int32_t(pointer)};
			int32_t bpos2{};
			while(data[pos]!=0xFF){
				if(pos>=data.length())throw std::runtime_error{"Unexpected end of data."};
				int commandType{data[pos]>>5};
				int length{(data[pos]&0x1F)+1};
				if(commandType==7){
					commandType=(data[pos]&0x1C)>>2;
					length=((int(data[pos])&3)<<8)+int(data[pos+1])+1;
					++pos;
				}
				if(bpos+length<0)throw std::runtime_error("Length ended up negative.");
				pos++;
				if(commandType>=4){
					bpos2=(int(data[pos])<<8)+int(data[pos+1]);
					if(bpos2<0)throw std::runtime_error("Reading negative data.");
					pos+=2;
				}
				switch(commandType){
					case UNCOMPRESSED_BLOCK:{
						bpos+=length;
						pos+=length;
					}break;
					case RUN_LENGTH_ENCODED_BYTE:{
						bpos+=length;
						++pos;
					}break;
					case RUN_LENGTH_ENCODED_SHORT:{
						if(bpos<0)throw std::runtime_error("Reading negative data while reading RLE short.");
						bpos+=2*length;
						pos+=2;
					}break;
					case INCREMENTAL_SEQUENCE:{
						bpos+=length;
						++pos;
					}break;
					case REPEAT_PREVIOUS_DATA:{
						if(bpos2<0)throw std::runtime_error("Reading negative data while repeating previous data.");
						bpos+=length;
					}break;
					case REVERSE_BITS:{
						if(bpos2<0)throw std::runtime_error("Reading negative data while reversing bits.");
						bpos+=length;
					}break;
					case UNKNOWN_1:{
						if(bpos2-length+1<0)throw std::runtime_error("Unexpected error in Unkonwn 1.");
						bpos+=length;
					}break;
					case UNKNOWN_2:{throw std::runtime_error("Reached unexpected command.");}break;
				}
			}
			return bpos;
		}
		std::vector<uint_fast16_t>decompress(){
			uint_fast32_t size{getCompressedSize(ptr)};
			if(size<1)throw std::runtime_error(std::format("Got an invalid data size of {}",size));
			std::vector<uint_fast16_t>blockOutput;
			blockOutput.resize(size);
			decompressBlock(ptr,blockOutput);
			return blockOutput;
		}
	private:
		void decompressBlock(uint_fast32_t ptrStart,std::vector<uint_fast16_t>&block){
			size_t maxLength{block.size()};
			int32_t pos{int32_t(ptrStart)};
			int32_t bpos{};
			int32_t bpos2{};
			int tmp;
			int read{};
			while(data[pos]!=0xFF){
				if (pos >= data.length())return;
				int commandType{data[pos]>>5};
				int len{(data[pos]&0x1F)+1};
				if(commandType==7){
					commandType=(int(data[pos])&0x1C)>>2;
					len=((int(data[pos])&3)<<8)+int(data[pos+1])+1;
					++pos;
				}
				if(bpos+len>maxLength||bpos+len<0)return;
				++pos;
				if(commandType>=4) {
					bpos2=(int(data[pos])<<8)+data[pos+1];
					if(bpos2>=maxLength||bpos2<0)return;
					pos+=2;
				}
				switch(commandType) {
					case UNCOMPRESSED_BLOCK:{
						while(len--!=0)block[bpos++]=data[pos++];
					}break;
					case RUN_LENGTH_ENCODED_BYTE:{
						while(len--!=0)block[bpos++]=data[pos];
						++pos;
					}break;
					case RUN_LENGTH_ENCODED_SHORT:{
						if(bpos+2*len>maxLength||bpos<0)return;
						while(len--!=0){
							block[bpos++]=data[pos];
							block[bpos++]=data[pos+1];
						}
						pos+=2;
					}break;
					case INCREMENTAL_SEQUENCE:{
						tmp=data[pos++];
						while(len--!=0)block[bpos++]=tmp++;
					}break;
					case REPEAT_PREVIOUS_DATA:{
						if(bpos2+len>maxLength||bpos2<0)return;
						for(int i:std::views::iota(0,len))block[bpos++]=block[bpos2+i];
					}break;
					case REVERSE_BITS:{
						if(bpos2+len>maxLength||bpos2<0)return;
						while(len--!=0)block[bpos++]=Rom::reversedBytes[block[bpos2++]&0xFF];
					}break;
					case UNKNOWN_1:{
						if(bpos2-len+1<0)return;
						while(len--!=0)block[bpos++]=block[bpos2--];
					}break;
					case UNKNOWN_2:{
						return;
					}break;
				}
			}
		}
	};

	struct BattleBackground{
		BattleBackground(std::u8string_view data,uint_fast32_t index){
			DataBlock readData{data,0xDCA1+index*uint_fast32_t(this->data.size())};
			for(int i=0;i<this->data.size();i++){
				this->data[i]=readData.readInt8();
			}
		};
		union{
			std::array<byte,17>data;
			struct{
				byte graphicsInd;
				byte paletteInd;
				byte bbp;
				byte paletteCycleType;
				byte paletteCycle1StartInd;
				byte paletteCycle1EndInd;
				byte paletteCycle2StartInd;
				byte paletteCycle2EndInd;
				byte paletteCycleSpeed;
				byte horizontalMovement;
				byte verticalMovement;
				byte horizontalAcceleration;
				byte verticalAcceleration;
				uint_fast32_t animationEffectsData;
			};
		};
		uint_fast32_t GetAnimation(){
			return (data[13]<<24)+(data[14]<<16)+(data[15]<<8)+data[16];
		}
	};
	struct BackgroundPalette{
		std::vector<Pixel>colors;
		byte bpp;
		uint_fast32_t addr;
		BackgroundPalette(std::u8string_view data,uint_fast16_t index,byte bpp)
		:bpp(bpp){
			DataBlock pointer{data,0xDAD9U+index*4U};
			uint_fast32_t addr=snesToHex(pointer.readInt32());
			DataBlock dataBlock{data,addr};
			this->addr=addr;
			if(bpp!=2&&bpp!=4)throw std::invalid_argument{std::format("Palette Error: Incorrect color depth specified. Must be 2 or 4, provided {}",bpp)};
			for(uint_fast8_t i:std::views::iota(0,pow(2,bpp))){
				uint_fast16_t clr16{dataBlock.readShort()};
				uint_fast8_t b{((clr16>>10)&31)*8U};
				uint_fast8_t g{((clr16>>5)&31)*8U};
				uint_fast8_t r{(clr16&31)*8U};
				colors.emplace_back(r,g,b);
			}
		}
	};
	using Tile=std::array<std::array<int,8>,8>;
	struct BackgroundGraphics{
		std::vector<uint_fast16_t>graphicsData;
		std::vector<uint_fast16_t>arrayGraphicsData;
		std::vector<Tile>tiles;
		byte bpp;
	private:
		void buildTiles(){
			int n{int(graphicsData.size())/(8*bpp)};
			for(int i{};i<n;i++){
				tiles.emplace_back();
				int o{i*8*bpp};
				for(int x{};x<8;x++){
					for(int y{};y<8;y++){
						int c{};
						for(int bp{};bp<bpp;bp++){
							int halfBp{bp/2};
							int gfx{int(graphicsData[o+y*2+(halfBp*16+(bp&1))])};
							c+=((gfx&(1<<7-x))>>7-x)<<bp;
						}
						tiles[i][x][y]=c;
					}
				}
			}
		}
	public:
		BackgroundGraphics(std::u8string_view data,uint_fast16_t index,byte bpp)
		:bpp(bpp){
			DataBlock graphicsPtr{data,0xD7A1U+index*4U};
			DataBlock loadGraphicsPtr{data,snesToHex(graphicsPtr.readInt32())};
			graphicsData=loadGraphicsPtr.decompress();
			buildTiles();
			DataBlock arrayPtrBlock{data,0xD93DU+index*4};
			uint_fast32_t arrayPtr{snesToHex(arrayPtrBlock.readInt32())};
			DataBlock arrayBlock{data,arrayPtr};
			arrayGraphicsData=arrayBlock.decompress();
		}
	};

	struct BackgroundLayer{

		struct DistortionEffect{
			enum{
				HORIZONTAL=1,
				HORIZONTAL_INTERLACED=2,
				VERTICAL=3
			};
			uint_fast16_t type(){
				return std::clamp(data[2],uint_fast8_t(1),uint_fast8_t(3U));
			}
			uint_fast16_t frequency(){
				return data[3]+(data[4]<<8);
			}
			uint_fast16_t amplitude(){
				return data[5]+(data[6]<<8);
			}
			uint_fast16_t compression(){
				return data[8]+(data[9]<<8);
			}
			uint_fast16_t frequencyAcceleration(){
				return data[10]+(data[11]<<8);
			}
			uint_fast16_t amplitudeAcceleration(){
				return data[12]+(data[13]<<8);
			}
			uint_fast16_t speed(){
				return data[14];
			}
			uint_fast16_t compressionAcceleration(){
				return data[15]+(data[16]<<8);
			}
			DistortionEffect(std::u8string_view data,uint16_t index){
				DataBlock block{data,0xF708U+index*17};
				for(int i:std::views::iota(0,17)){
					this->data[i]=block.readInt8();
				}
			}
		private:
			std::array<uint_fast8_t,17>data;
		};

		struct Distorter{
			const float c1{1/512.f};
			const float c2{8*PI/(1024*256.f)};
			const float c3{PI/60.f};
			Sprite&bitmap;
			std::unique_ptr<DistortionEffect>effect;
			float amplitude,frequency,compression,speed;
			Distorter(Sprite&bitmap)
			:bitmap(bitmap){}
			void setup(int ticks){
				int t2{ticks*2};
				amplitude=c1*(effect->amplitude()+effect->amplitudeAcceleration()*t2);
				frequency=c2*(effect->frequency()+(effect->frequencyAcceleration()*t2));
				compression=1+(effect->compression()+(effect->compressionAcceleration()*t2))/256.f;
				speed=c3*speed*ticks;
			}
			float getAppliedOffset(int y){
				float s=std::round(amplitude*sin(frequency*y+speed));
				switch(effect->type()){
					case DistortionEffect::HORIZONTAL:return s;
					case DistortionEffect::HORIZONTAL_INTERLACED:return y%2?s:-s;
					case DistortionEffect::VERTICAL:return fmod(floor(s+y*effect->compression()),256.f);
				}
			}
			void computeFrame(PixelGameEngine*pge,int ticks,float alpha,bool erase){
				setup(ticks);
				for(int y:std::views::iota(0,pge->ScreenHeight())){
					float offset{getAppliedOffset(y)};
					int L=effect->type()==DistortionEffect::VERTICAL?offset:y;
					for(int x:std::views::iota(0,pge->ScreenWidth())){
						int dx=x;
						if(effect->type()==DistortionEffect::HORIZONTAL||effect->type()==DistortionEffect::HORIZONTAL_INTERLACED)dx=fmod(x+offset,pge->ScreenWidth());
						if(erase){
							Pixel newCol{bitmap.GetPixel(dx,L)};
							newCol.r*=alpha;
							newCol.g*=alpha;
							newCol.b*=alpha;
							pge->Draw(dx,L,newCol);
						}else{
							Pixel newCol{bitmap.GetPixel(dx,L)};
							newCol.r*=alpha;
							newCol.g*=alpha;
							newCol.b*=alpha;
							Pixel mixedCol{pge->GetDrawTarget()->GetPixel(dx,L)};
							mixedCol.r+=newCol.r;
							mixedCol.g+=newCol.g;
							mixedCol.b+=newCol.b;
							pge->Draw(dx,L,mixedCol);
						}
					}
				}
			}
		};

		struct PaletteCycle{
			std::vector<Pixel>originalCols;
			std::vector<Pixel>currentCols;
			byte type,start1,end1,start2,end2,speed,cycleCountdown,cycleCount{};
			PaletteCycle(BattleBackground&background,BackgroundPalette&palette)
			:type(background.paletteCycleType),start1(background.paletteCycle1StartInd),end1(background.paletteCycle1EndInd),start2(background.paletteCycle2StartInd),end2(background.paletteCycle2EndInd),speed(background.paletteCycleSpeed/2),cycleCountdown(speed),originalCols(palette.colors),currentCols(originalCols){
				for(int i:std::views::iota(0U,originalCols.size())){
					originalCols.emplace_back(originalCols[i]);
				}
			}
			void cycle(){
				if(speed==0)return;
				cycleCountdown=std::clamp(cycleCountdown-1,0,int(std::numeric_limits<byte>::max()));
				if(cycleCountdown<=0){
					cycleColors();
					cycleCount++;
					cycleCountdown=speed;
				}
			}
			void cycleColors(){
				if(type==1||type==2){
					int cycleLength{end1-start1+1};
					int cycle1Position{cycleCount%cycleLength};
					for(int i:std::views::iota(int(start1),end1+1)){
						int newColor=i-cycle1Position;
						if(newColor<start1)newColor+=cycleLength;
						currentCols[i]=originalCols[newColor];
					}
				}
				if(type==2){
					int cycleLength{end2-start2+1};
					int cycle2Position{cycleCount%cycleLength};
					for(int i:std::views::iota(int(start2),end2+1)){
						int newColor=i-cycle2Position;
						if(newColor<start2)newColor+=cycleLength;
						currentCols[i]=originalCols[newColor];
					}
				}
				if(type==3){
					int cycleLength{end1-start1+1};
					int cycle1Position{cycleCount%(cycleLength*2)};
					for(int i:std::views::iota(int(start1),end1+1)){
						int newColor=i+cycle1Position;
						if(newColor>end1){
							int difference{newColor-end1-1};
							newColor=end1-difference;
							if(newColor<start1){
								difference=start1-newColor-1;
								newColor=start1+difference;
							}
						}
						currentCols[i]=originalCols[newColor];
					}
				}
			}
		};

		PixelGameEngine*pge;
		Sprite*spr;
		Distorter distorter;
		BattleBackground&background;
		BackgroundGraphics&graphics;
		PaletteCycle cycle;
		uint_fast16_t backgroundInd;

		void drawTile(const int x,const int y,const int tile,const int subPalette,const bool verticalFlip,const bool horizontalFlip){
			int px{},py{};
			for(int i:std::views::iota(0,8)){
				if(horizontalFlip)px=x+7-i;
				else px=x+i;
				for(int j:std::views::iota(0,8)){
					if(verticalFlip)py=y+7-j;
					else py=y+j;
					spr->SetPixel(px,py,cycle.currentCols[graphics.tiles[tile][i][j]]);
				}
			}
		}

		void draw(){
			int block{};
			int tile{};
			int subPalette{}; //Subpalette should always be zero....I don't understand?
			int n{};
			int b1{};
			int b2{};
			bool verticalFlip{false};
			bool horizontalFlip{false};
			for(int i:std::views::iota(0,32)){
				for(int j:std::views::iota(0,32)){
					n=j*32+i;
					b1=graphics.arrayGraphicsData[n*2];
					b2=graphics.arrayGraphicsData[n*2+1]<<8;
					block=b1+b2;
					tile=block&0x3FF;
					verticalFlip=block&0x8000;
					horizontalFlip=block&0x4000;
					//subPalette=(block>>10)&7; //THIS SHOULD NOT BE NECESSARY!!!
					drawTile(i*8,j*8,tile,0,verticalFlip,horizontalFlip);
				}
			}
		}
		BackgroundLayer(PixelGameEngine*pge,uint_fast16_t backgroundInd,std::vector<BattleBackground>&backgrounds,std::vector<BackgroundPalette>&palettes,std::vector<BackgroundGraphics>&graphics,std::u8string_view data)
		:pge(pge),spr(new Sprite(256,256)),distorter(*spr),backgroundInd(backgroundInd),background(backgrounds[backgroundInd]),graphics(graphics[background.graphicsInd]),cycle(background,palettes[background.paletteInd]){
			uint_fast32_t effectVal{((background.GetAnimation()>>16)&0xFF)};
			if(effectVal==0)effectVal=(((background.GetAnimation()>>24)&0xFF));
			distorter.effect=std::make_unique<DistortionEffect>(data,effectVal);
		}
	};
	std::u8string data;
	inline static std::string reversedBytes;
	std::vector<BattleBackground>backgrounds;
	std::vector<BackgroundPalette>palettes;
	std::vector<BackgroundGraphics>graphics;

	const BackgroundLayer*const GetLayer1(){return layer1.get();}
	const BackgroundLayer*const GetLayer2(){return layer2.get();}
	void SetLayer1(uint_fast16_t backgroundInd){
		backgroundInd=std::clamp(backgroundInd,0U,uint_fast16_t(backgrounds.size()));
		layer1=std::make_unique<BackgroundLayer>(nullptr,backgroundInd,backgrounds,palettes,graphics,data);
	}
	void SetLayer2(uint_fast16_t backgroundInd){
		backgroundInd=std::clamp(backgroundInd,0U,uint_fast16_t(backgrounds.size()));
		layer2=std::make_unique<BackgroundLayer>(nullptr,backgroundInd,backgrounds,palettes,graphics,data);
	}
	void AdvanceLayers(int tick,float alpha1,float alpha2){
		layer1->cycle.cycle();
		layer1->draw();
		layer1->distorter.computeFrame(pge,tick,alpha1,true);
		layer2->cycle.cycle();
		layer2->draw();
		layer2->distorter.computeFrame(pge,tick,alpha2,false);
	}
private:
	std::unique_ptr<BackgroundLayer>layer1,layer2;
	PixelGameEngine*pge;
public:
	Rom(PixelGameEngine*pge)
	:pge(pge){
		#pragma region Setup Reversed Bytes
		reversedBytes.resize(256);
		for(int i:std::views::iota(0U,reversedBytes.size())){
			uint_fast8_t newNumb{};
			for(int digit=0;digit<8;digit++){
				uint_fast8_t digitVal{1U<<digit};
				if(i&digitVal)newNumb|=1<<(7-digit);
			}
			reversedBytes[i]=newNumb;
			i++;
		}
		#pragma endregion

		std::ifstream dataStream{"truncated_backgrounds.dat",std::ios_base::binary};
		while(dataStream.good())data+=dataStream.get();

		for(uint_fast16_t i:std::views::iota(0U,MAX_INDEX+1U)){
			backgrounds.emplace_back(data,i);
			paletteBits[backgrounds.back().paletteInd]=backgrounds.back().bbp;
			graphicsBits[backgrounds.back().graphicsInd]=backgrounds.back().bbp;
		}
		for(uint_fast16_t i:std::views::iota(0U,paletteBits.size())){
			palettes.emplace_back(data,i,paletteBits[i]);
		}
		for(uint_fast16_t i:std::views::iota(0U,graphicsBits.size())){
			graphics.emplace_back(data,i,graphicsBits[i]);
		}

		layer1=std::make_unique<BackgroundLayer>(pge,0U,backgrounds,palettes,graphics,data);
		layer2=std::make_unique<BackgroundLayer>(pge,0U,backgrounds,palettes,graphics,data);
	}
};

// Override base class with your custom functionality
class EarthboundBattleBackgrounds : public olc::PixelGameEngine
{
public:
	EarthboundBattleBackgrounds()
	{
		// Name your application
		sAppName = "Earthbound Battle Backgrounds";
	}

	Rom*rom;

public:
	bool OnUserCreate() override
	{
		rom=new Rom(this);
		return true;
	}

	void PaletteDisplayTest(){
		if(GetMouseWheel()<0)yOffset-=32;
		if(GetMouseWheel()>0)yOffset+=32;

		for(uint_fast8_t row=0;Rom::BackgroundPalette&palette:rom->palettes){
			for(uint_fast8_t index=0;Pixel&col:palette.colors){
				FillRectDecal(vf2d{index*8.f+8,row*8.f+yOffset},{8,8},col);
				index++;
			}
			row++;
		}
	}

	uint_fast8_t selectedPalette{};

	void GraphicsDisplayTest(){
		if(GetMouseWheel()<0)yOffset-=32;
		if(GetMouseWheel()>0)yOffset+=32;

		Clear(BLACK);

		DrawStringDecal({},std::format("Selected Palette: {}",selectedPalette));

		if(GetKey(PGUP).bPressed)selectedPalette=std::clamp(int(selectedPalette)-1,0,int(rom->palettes.size()-1));
		if(GetKey(PGDN).bPressed)selectedPalette=std::clamp(int(selectedPalette)+1,0,int(rom->palettes.size()-1));;

		Rom::BackgroundPalette&palette{rom->palettes[selectedPalette]};
		for(uint_fast8_t index=0;Pixel&col:palette.colors){
			FillRectDecal(vf2d{index*8.f,8.f},{8,8},col);
			index++;
		}
		DrawRectDecal(vf2d{8.f,8.f},{8*16,8});

		for(uint_fast8_t row=0;Rom::BackgroundGraphics&graphics:rom->graphics){
			for(uint_fast8_t col=0;Rom::Tile&tile:graphics.tiles){
				for(int x{};x<tile.size();x++){
					for(int y{};y<tile[x].size();y++){
						int paletteInd{tile[x][y]};
						Pixel color{MAGENTA};
						const bool ColorInRange{paletteInd<palette.colors.size()};
						if(ColorInRange)color=palette.colors[paletteInd];
						Draw(vi2d{col*8,row*8+yOffset+20}+vi2d{x,y},color);
					}
				}
				col++;
			}
			row++;
		}
	}

	int yOffset{0};
	float accumulatedTime{};
	int tick{};

	uint16_t selectedLayer1{};
	uint16_t selectedLayer2{};

	void runTick(){
		float alpha1{1.f},alpha2{1.f};
		if(rom->GetLayer1()->backgroundInd&&!rom->GetLayer2()->backgroundInd){
			alpha1=1.f;
			alpha2=0.f;
		}
		if(!rom->GetLayer1()->backgroundInd&&rom->GetLayer2()->backgroundInd){
			alpha1=0.f;
			alpha2=1.f;
		}
		rom->AdvanceLayers(tick,alpha1,alpha2);
	}

	bool OnUserUpdate(float fElapsedTime) override
	{
		if(GetKey(RIGHT).bPressed){
			selectedLayer1=std::clamp(selectedLayer1+1,0,int(rom->backgrounds.size()));
			rom->SetLayer1(selectedLayer1);
		}
		if(GetKey(LEFT).bPressed){
			selectedLayer1=std::clamp(selectedLayer1-1,0,int(rom->backgrounds.size()));
			rom->SetLayer1(selectedLayer1);
		}
		if(GetKey(UP).bPressed){
			selectedLayer2=std::clamp(selectedLayer2+1,0,int(rom->backgrounds.size()));
			rom->SetLayer2(selectedLayer2);
		}
		if(GetKey(DOWN).bPressed){
			selectedLayer2=std::clamp(selectedLayer2-1,0,int(rom->backgrounds.size()));
			rom->SetLayer2(selectedLayer2);
		}

		accumulatedTime+=fElapsedTime;
		while(accumulatedTime>=1/60.f){
			runTick();
			accumulatedTime-=1/60.f;
			tick++;
		}

		DrawStringDecal({},std::format("Layer 1: {}   Layer 2: {}",selectedLayer1,selectedLayer2));
		return true;
	}
};

int main()
{
	EarthboundBattleBackgrounds demo;
	if (demo.Construct(256, 224, 4, 4))
		demo.Start();
	return 0;
}